package com.kdanmobile.admanager.admob

import android.content.Context
import android.view.View
import com.google.android.gms.ads.AdRequest
import com.google.android.gms.ads.appopen.AppOpenAd
import com.google.android.gms.ads.appopen.AppOpenAdPresentationCallback
import com.google.android.gms.ads.appopen.AppOpenAdView
import com.kdanmobile.admanager.AppOpenAdListener
import com.kdanmobile.admanager.AppOpenAdManager
import java.util.concurrent.ConcurrentHashMap
import java.util.concurrent.CopyOnWriteArrayList
import java.util.concurrent.CopyOnWriteArraySet

class AdMobAppOpenAdManager(private val context: Context) : AppOpenAdManager {

    private val listeners = CopyOnWriteArrayList<AppOpenAdListener>()
    private val loadingAdSet = CopyOnWriteArraySet<String>()
    private val loadedAdMap = ConcurrentHashMap<String, AppOpenAd>()
    override var isPersonalized = true

    override fun registerAppOpenAdListener(appOpenAdListener: AppOpenAdListener) {
        listeners += appOpenAdListener
    }

    override fun unregisterAppOpenAdListener(appOpenAdListener: AppOpenAdListener) {
        listeners -= appOpenAdListener
    }

    @Synchronized
    override fun requestAppOpenAd(adUnitId: String, orientation: AppOpenAdManager.Orientation) {
        if (isAppOpenAdLoaded(adUnitId)) return
        if (isAppOpenAdLoading(adUnitId)) return
        loadingAdSet += adUnitId
        AppOpenAd.load(
                context,
                adUnitId,
                RequestFactory.create(isPersonalized),
                when (orientation) {
                    AppOpenAdManager.Orientation.PORTRAIT -> AppOpenAd.APP_OPEN_AD_ORIENTATION_PORTRAIT
                    AppOpenAdManager.Orientation.LANDSCAPE -> AppOpenAd.APP_OPEN_AD_ORIENTATION_LANDSCAPE
                },
                object : AppOpenAd.AppOpenAdLoadCallback() {
                    override fun onAppOpenAdLoaded(appOpenAd: AppOpenAd?) {
                        super.onAppOpenAdLoaded(appOpenAd)
                        loadingAdSet -= adUnitId
                        appOpenAd?.let {
                            loadedAdMap.put(adUnitId, it)
                        }
                        listeners.forEach { it.onAppOpenAdLoaded(adUnitId) }
                    }

                    override fun onAppOpenAdFailedToLoad(p0: Int) {
                        super.onAppOpenAdFailedToLoad(p0)
                        loadingAdSet -= adUnitId
                        listeners.forEach { it.onAppOpenAdFailedToLoad(adUnitId) }
                    }
                }
        )
    }

    override fun takeAppOpenAdView(adUnitId: String, onClosed: () -> Unit): View? {
        val ad = loadedAdMap[adUnitId] ?: return null
        return AppOpenAdView(context).apply {
            setAppOpenAd(ad)
            setAppOpenAdPresentationCallback(object : AppOpenAdPresentationCallback() {
                override fun onAppOpenAdClosed() {
                    super.onAppOpenAdClosed()
                    onClosed.invoke()
                }
            })
        }
    }

    override fun isAppOpenAdLoading(adUnitId: String): Boolean {
        return loadingAdSet.contains(adUnitId)
    }

    override fun isAppOpenAdLoaded(adUnitId: String): Boolean {
        return loadedAdMap.contains(adUnitId)
    }
}