package com.kdanmobile.admanager.admob

import android.app.Activity
import android.content.Context
import android.view.View
import com.google.android.gms.ads.MobileAds
import com.kdanmobile.admanager.*
import java.util.concurrent.CopyOnWriteArrayList

class AdmobAdManager(
        private val context: Context,
        private val adMobAppOpenAdManager: AdMobAppOpenAdManager = AdMobAppOpenAdManager(context)
) :
        AdManager,
        AppOpenAdManager by adMobAppOpenAdManager {

    private lateinit var interstitialAdManager: InterstitialAdManager
    private lateinit var nativeAdManager: NativeAdManager
    override var isInitialized = false
    override val onInitListeners = CopyOnWriteArrayList<Runnable>()
    override var isPersonalized: Boolean = true
        set(value) {
            nativeAdManager?.isPersonalized = value
            interstitialAdManager?.isPersonalized = value
            adMobAppOpenAdManager.isPersonalized = value
        }

    override fun init(context: Context, adUnitId: String, volume: Float) {

        MobileAds.initialize(context) {
            isInitialized = true
            onInitListeners.forEach { it.run() }
        }
        MobileAds.setAppVolume(volume)

        interstitialAdManager = AdmobInterstitialAdManager()
        nativeAdManager = AdmobNativeAdManager(context)
    }

    override fun requestInterstitial(adUnitId: String) {
        interstitialAdManager.request(adUnitId)
    }

    override fun requestInterstitial(activity: Activity, adUnitId: String) {
        interstitialAdManager.request(activity, adUnitId)
    }

    override fun showInterstitial(adUnitId: String): Boolean {
        return interstitialAdManager.show(adUnitId)
    }

    override fun destroyInterstitial(activity: Activity) {
        interstitialAdManager.destroy(activity)
    }

    override fun registerInterstitialAdListener(interstitialAdListener: InterstitialAdListener) {
        interstitialAdManager.registerListener(interstitialAdListener)
    }

    override fun unregisterInterstitialAdListener(interstitialAdListener: InterstitialAdListener) {
        interstitialAdManager.unregisterListener(interstitialAdListener)
    }

    override fun isInterstitialLoading(adUnitId: String): Boolean {
        return interstitialAdManager.isLoading(adUnitId)
    }

    override fun isInterstitialLoaded(adUnitId: String): Boolean {
        return interstitialAdManager.isLoaded(adUnitId)
    }

    override fun registerNativeAdListener(nativeAdListener: NativeAdListener) {
        nativeAdManager.listener += nativeAdListener
    }

    override fun unregisterNativeAdListener(nativeAdListener: NativeAdListener) {
        nativeAdManager.listener -= nativeAdListener
    }

    override fun requestNative(adUnitId: String, nativeAdViewBinder: NativeAdViewBinder) {
        nativeAdManager.request(adUnitId, nativeAdViewBinder)
    }

    override fun takeNativeAdView(adUnitId: String, nativeAdViewBinder: NativeAdViewBinder): View? {
        return nativeAdManager.takeAdView(adUnitId, nativeAdViewBinder)
    }

    override fun isNatvieLoading(adUnitId: String): Boolean {
        return nativeAdManager.isLoading(adUnitId)
    }
}